@@ -19,7 +19,7 @@ from pywss.closing import Closing
 from pywss.routing import Route
 from pywss.openapi import openapi_ui_template, merge_dict, parameters_filter
 
-__version__ = '0.1.9'
+__version__ = '0.1.12'
 
 
 class Context:
@@ -50,6 +50,7 @@ class Context:
         self.response_headers: dict = {
             "Server": "Pywss",
             "PywssVersion": __version__,
+            "Content-Length": 0,
         }
         self.response_body: list = []
 
@@ -169,6 +170,8 @@ class Context:
         self.response_status_code = status_code
 
     def redirect(self, url, status_code=StatusFound) -> None:
+        if "?" not in url and "?" in self.path:
+            url = f"{url}?{self.path.split('?', 1)[1]}"
         self.set_status_code(status_code)
         self.set_header("Location", url)
 
@@ -246,7 +249,9 @@ class App:
 
     def register(self, method, route, handlers) -> None:
         route = f"/{route.strip().strip('/')}" if route else route
-        self.full_match_routes[f"{method}{self.base_route}{route}"] = list(self.base_handlers) + list(handlers)
+        route = f"{self.base_route}{route}"
+        route = f"/{route.strip().strip('/')}"
+        self.full_match_routes[f"{method}{route}"] = list(self.base_handlers) + list(handlers)
 
     def build(self) -> None:
         routes = {}
@@ -321,6 +326,8 @@ class App:
         self.base_handlers += list(handlers)
 
     def static(self, route, rootDir, *handlers, staticHandler=NewStaticHandler) -> None:
+        if not os.path.exists(rootDir):
+            raise Exception(f"not found {rootDir}")
         if staticHandler:
             handlers = list(handlers)
             handlers.append(staticHandler(rootDir))
@@ -393,7 +400,8 @@ class App:
                 if err:
                     request.sendall(b"HTTP/1.1 400 BadRequest\r\n")
                     return
-                log = log.update(method=method, path=path)
+                route = f"{method.upper()}/{path.split('?', 1)[0].strip('/')}"
+                log = log.update(route=route)
                 hes, err = parse_headers(rfd)
                 if err:
                     request.sendall(b"HTTP/1.1 400 BadRequest\r\n")
@@ -404,7 +412,6 @@ class App:
                     break
                 # full match
                 paths = {}
-                route = f"{method.upper()}/{path.split('?', 1)[0].strip('/')}"
                 handlers = self.full_match_routes.get(route, None)
                 # parser match
                 if not handlers:
@@ -445,28 +452,29 @@ class App:
 
     def run(self, host="0.0.0.0", port=8080, grace=0, request_queue_size=5, poll_interval=0.5) -> None:
         self.build()
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sock.bind((host, port))
-        sock.listen(request_queue_size)
-        selector = selectors.PollSelector if hasattr(selectors, 'PollSelector') else selectors.SelectSelector
-        with self.log.trycache():
-            with selector() as _selector:
-                _selector.register(sock.fileno(), selectors.EVENT_READ)
-                self.log. \
-                    update(version=__version__). \
-                    variables(host, port, grace). \
-                    info("server start")
-                while self.running:
-                    ready = _selector.select(poll_interval)
-                    if not ready:
-                        continue
-                    request, address = sock.accept()
-                    threading.Thread(target=self._, args=(request, address)).start()
-        for i in range(grace):
-            self.log.update(hit=i + 1, grace=grace).warning("server closing")
-            time.sleep(1)
-        self.log.warning("server closed")
-        sock.close()
+        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
+            sock.bind((host, port))
+            sock.listen(request_queue_size)
+            selector = selectors.PollSelector if hasattr(selectors, 'PollSelector') else selectors.SelectSelector
+            with self.log.trycache():
+                with selector() as _selector:
+                    _selector.register(sock.fileno(), selectors.EVENT_READ)
+                    self.log.update(
+                        version=__version__,
+                        host=host,
+                        port=port,
+                        grace=grace
+                    ).info("server start")
+                    while self.running:
+                        ready = _selector.select(poll_interval)
+                        if not ready:
+                            continue
+                        request, address = sock.accept()
+                        threading.Thread(target=self._, args=(request, address)).start()
+            for i in range(grace):
+                self.log.update(hit=i + 1, grace=grace).warning("server closing")
+                time.sleep(1)
+            self.log.warning("server closed")
 
 
 class data(dict):
